/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <dlog.h>
#include <app.h>
#include <metadata_extractor.h>
#include <tizen.h>
#include <Elementary.h>
#include <efl_extension.h>
#include <app_preference.h>


#include "$(appName).h"
#include "view.h"
#include "data.h"

#define SAVED_MEMO_INDEX "saved_memo_index"
#define EXTENSION ".3gp"

typedef enum {
	YEAR = 0,
	MON,
	DAY,
	HOUR,
	MIN,
	SEC,
	DUMIN,
	DUSEC,
} date_type_e;

static struct data_info {
	Eina_List *voice_list;
	voice_list_item_info *selected_item;
	int will_be_saved_index;
} s_info = {
	.voice_list = NULL,
	.selected_item = NULL,
	.will_be_saved_index = 0,
};

void data_initialize(void)
{
	/*
	 * If you need to initialize managing data,
	 * please use this function.
	 */
}

void data_finalize(void)
{
	/*
	 * If you need to finalize managing data,
	 * please use this function.
	 */
}

const char *data_get_text(void)
{
	/*
	 * You can use this function to retrieve data.
	 */
	return NULL;
}

/*
 * @brief: Store the index for next launch
 */
void data_store_index(void)
{
	preference_set_int(SAVED_MEMO_INDEX, s_info.will_be_saved_index);
}

/*
 * @brief: Increase the index for next saving
 */
void data_increase_index(void)
{
	s_info.will_be_saved_index++;
}

/*
 * @brief: Append item to voice list
 * @param[record_info]: The item recorded
 */
void data_append_item_info_to_list(voice_list_item_info *record_info)
{
	s_info.voice_list = eina_list_append(s_info.voice_list, record_info);
}

/*
 * @brief: Get file title according to the index
 */
char *data_get_recording_file_title_according_to_index(void)
{
	char file_title[256];

	snprintf(file_title, sizeof(file_title), "Voice %02i", s_info.will_be_saved_index);
	return strdup(file_title);
}

/*
 * @brief: Get index saved before exit this app
 */
void data_get_index(void)
{
	bool is_exist = false;
	int count = 0;

	/* Check whether index is saved or not */
	int err = preference_is_existing(SAVED_MEMO_INDEX, &is_exist);
	if (err != PREFERENCE_ERROR_NONE) {
	}

	if (is_exist) {
		preference_get_int(SAVED_MEMO_INDEX, &count);
	} else {
		/* This means that no voice memo is saved yet */
		count++;
	}

	s_info.will_be_saved_index = count;
}

/*
 * @brief: Get file name according to the index and absolute path
 */
char *data_get_filename_according_to_count(void)
{
	char filename[255] = {'\0' };
	snprintf(filename, sizeof(filename), "%sVoice %02i.3gp", app_get_data_path(), s_info.will_be_saved_index);

	return strdup(filename);
}

/*
 * @brief: Set the selected item
 * @param[selected_item]: The selected item
 */
void data_set_selected_item(voice_list_item_info *selected_item)
{
	s_info.selected_item = selected_item;
}

/*
 * @brief: Get the voice list
 */
Eina_List *data_get_voice_list(void)
{
	return s_info.voice_list;
}

/*
 * @brief: Get the number of items in the voice list
 */
int data_get_voice_list_item_count(void)
{
	return eina_list_count(s_info.voice_list);
}

/*
 * @brief: Get recording information using file path
 * @param[file_name]: The name of the recorded file
 * @param[file_path]: The path of the file
 */
voice_list_item_info *data_get_recording_info_from_file_path(char *file_name, char *file_path)
{
	metadata_extractor_h metadata;
	voice_list_item_info *item_info = NULL;
	char dummy[256];
	char *value;
	char *date;
	int date_array[8];
	int duration;

	/* allocate memory for saving the information */
	item_info = malloc(sizeof(voice_list_item_info));

	/* Create metadata handle */
	metadata_extractor_create(&metadata);

	/*
	 * This function is called under two situations
	 * One is when app get the information from DB, just know file name
	 * The other is when recorder just saved the file, in this case, just know file path
	 */
	if (file_name && file_path == NULL) {
		/* attach absolute path to the file name */
		item_info->absolute_file_path = data_get_absolute_path_to_filename(file_name);
		/*
		 * Sets the buffer to extract the information
		 * Must be absolute full path of the file from which you want to get the information
		 */
		metadata_extractor_set_path(metadata, item_info->absolute_file_path);
	} else if (file_path) {
		metadata_extractor_set_path(metadata, file_path);
		item_info->absolute_file_path = file_path;
	}
	/* Get the playing time from the file */
	metadata_extractor_get_metadata(metadata, METADATA_DURATION, &value);
	/* Get the information when this file is saved */
	metadata_extractor_get_metadata(metadata, METADATA_RECDATE, &date);

	/*
	 * To convert the information to the form that we want to show,
	 * divide into several smallest units.
	 */
	sscanf(date, "%d-%d-%d %d:%d:%d", &date_array[YEAR], &date_array[MON], &date_array[DAY], &date_array[HOUR], &date_array[MIN], &date_array[SEC]);

	/* Convert string to int to calculate the time */
	sscanf(value, "%d", &duration);

	date_array[DUMIN] = (duration / PER_MSEC) / PER_MIN;
	date_array[DUSEC] = (duration / PER_MSEC) % PER_MIN;


	if (file_name == NULL) {
		/*
		 * File name is 'NULL' means that recorder just saved the file
		 * In this case to extract 'title' of the file, use index
		 */
		item_info->title = data_get_recording_file_title_according_to_index();

	} else {
		/*
		 * File name is not 'NULL' means that app get the file from the DB on launch
		 * In this case to extract 'title' of the file, remove the extension
		 */
		char a[10];
		int b;
		char c[10];
		char title[256];

		/* divide into three smallest units */
		sscanf(file_name, "%s %02i.%s", a, &b, c);

		/* Sum these units without extension */
		snprintf(title, sizeof(title), "%s %02i", a, b);

		item_info->title = strdup(title);

	}

	/* Make date shown at the detail view */
	snprintf(dummy, sizeof(dummy), "%02i/%02i/%04i %02i:%02i", date_array[DAY], date_array[MON], date_array[YEAR], date_array[DUMIN], date_array[DUSEC]);
	item_info->detail_date = strdup(dummy);

	/* Display the time as a form of "11:34" */
	if (date_array[HOUR] > 12) {
		snprintf(dummy, sizeof(dummy), "%02i:%02i PM", date_array[HOUR]-12, date_array[MIN]);
	} else if (date_array[HOUR]  == 12) {
		snprintf(dummy, sizeof(dummy), "%02i:%02i PM", date_array[HOUR], date_array[MIN]);
	} else {
		snprintf(dummy, sizeof(dummy), "%02i:%02i AM", date_array[HOUR], date_array[MIN]);
	}
	item_info->detail_time = strdup(dummy);

	snprintf(dummy, sizeof(dummy), "%02i:%02i", date_array[DUMIN], date_array[DUSEC]);
	item_info->duration = strdup(dummy);

	snprintf(dummy, sizeof(dummy), "%02i/%02i", date_array[MON], date_array[DAY]);
	item_info->date = strdup(dummy);

	/*
	 * Set the style for the shape will be shown on the Genlist
	 * See the '_set_genlist_item_class' function in the 'view.c'
	 */
	item_info->style = strdup("3text");

	return item_info;
}

/*
 * @brief: Check whether extension is same or not
 * @param[file_name]: The subject to be checked
 */
Eina_Bool data_check_file_extension(char *file_name)
{
	int length = 0;
	int is_true = 0;

	length = strlen(file_name);

	/* You customize the extension with another extension instead of "3gp" */
	is_true = strcmp(file_name + (length - 4), EXTENSION);

	return is_true;
}

/*
 * @brief: Get the voice files from DB and
 * convert its' information to the Structure and make a list for management
 */
void data_create_voice_list_from_db(void)
{
	DIR *handle;
	struct dirent *entry;
	voice_list_item_info *item_info = NULL;

	/* Open directory which has saved voice memo file */
	handle = opendir(app_get_data_path());

	/* Search all the file of the directory to find voice memo file */
	while ((entry = readdir(handle))) {
		if (!strcmp(entry->d_name, "..")) {
			continue;
		} else if (!strcmp(entry->d_name, ".")) {
			continue;
			/* Check whether the extension is same with what you want to find */
		} else if (!data_check_file_extension(entry->d_name)) {
			/* Using file name, get more information and save as a structure */
			item_info = data_get_recording_info_from_file_path(entry->d_name, NULL);
			/* Adds the item_info to the voice list */
			s_info.voice_list = eina_list_append(s_info.voice_list, item_info);
		}
	}
	closedir(handle);
}

/*
 * @brief: Release the resource of the voice list
 */
void data_release_voice_list(void)
{
	voice_list_item_info *data;

	EINA_LIST_FREE(s_info.voice_list, data) {
		free(data);
	}
}

/*
 * @brief: Adds a absolute path of the file to the file name
 * @param[name]: File name
 */
char *data_get_absolute_path_to_filename(char *name)
{
	char full_name[256];

	snprintf(full_name, sizeof(full_name), "%s/%s", app_get_data_path(), name);
	return strdup(full_name);
}

/*
 * @brief: Remove selected voice memo information from the list and delete the file from DB
 */
void data_remove_selected_item_from_list_and_db(void)
{
	Eina_List *l;
	Eina_List *ln;
	voice_list_item_info *list_data;

	EINA_LIST_FOREACH_SAFE(s_info.voice_list, l, ln, list_data) {
		if (!strcmp(list_data->title, s_info.selected_item->title)) {
			/* Remove voice memo from voice memo list */
			s_info.voice_list = eina_list_remove_list(s_info.voice_list, l);
			/* Delete the selected voice memo file from the DB */
			if (unlink(list_data->absolute_file_path) == 0) {
				dlog_print(DLOG_INFO, LOG_TAG, "Failed to unlink the file");
			};

			/*
			 * Remove from the genlist
			 * After this function, genlist update the list automatically
			 */
			elm_object_item_del(s_info.selected_item->item);
			free(s_info.selected_item);

			break;
		}
	}
}

/*
 * @brief: Get the number of items in the list
 */
int data_count_voice_list_item(void)
{
	int count = 0;

	count = eina_list_count(s_info.voice_list);

	return count;
}

/*
 * @brief: Get path of resource
 * @param[file_in]: File name
 * @param[file_path_out]: The point to which save full path of the file
 * @param[file_path_max]: Size of file name include path
 */
void data_get_resource(const char *file_in, char *file_path_out, int file_path_max)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(file_path_out, file_path_max, "%s%s", res_path, file_in);
		free(res_path);
	}
}

/*
 * @brief: Get text to display on popup
 */
char *data_get_popup_text(void)
{
	return strdup("Recording fail!!");
}
